/**********************************************************************\
 © COPYRIGHT 2013 Corporation for National Research Initiatives (CNRI);
                        All rights reserved.
               
        The HANDLE.NET software is made available subject to the
         Handle System Public License, which may be obtained at
         http://hdl.handle.net/4263537/5030 or hdl:4263537/5030
\**********************************************************************/

package net.handle.apps.servlet_proxy;

import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.net.*;

@SuppressWarnings("deprecation")
public class OpenURL
  extends HttpServlet
  implements SingleThreadModel
{

  // normal HDLProxy is used for resolution after the url is parsed
  HDLProxy hdlproxy = new HDLProxy();

  static final String URL_TYPE[] = {"URL"};
  static final Hashtable passThroughKeys = new Hashtable();
  static {
    passThroughKeys.put("nosfx", "");
    passThroughKeys.put("nols", "");
  }

  public void init() throws ServletException {
    hdlproxy.init(getServletConfig());
  }


  private final void doOpenURLResolution(HttpServletRequest req,
                                         HttpServletResponse resp)
    throws ServletException, IOException
  { 
    //Hashtable params = new Hashtable();
    int ampIdx = -1;
    int sectionStart; 
    int eqIdx;
    String query = req.getQueryString();
    
    HDLServletRequest hdlReq = new HDLServletRequest(hdlproxy, req, resp, HDLProxy.resolver);
    hdlReq.params.clear();
    
    String paramKey = null;
    String paramVal = null;
    String hdl = null;
    do {
      sectionStart = ampIdx+1;
      ampIdx = query.indexOf('&', sectionStart);
      String section = (ampIdx<0) ? query.substring(sectionStart) :
                                    query.substring(sectionStart,ampIdx);
      eqIdx = section.indexOf('=');
      if(eqIdx<0) {
        paramKey = URLDecoder.decode(section,"UTF-8");
        paramVal = "";
      } else {
        paramKey = URLDecoder.decode(section.substring(0,eqIdx),"UTF-8");
        paramVal = URLDecoder.decode(section.substring(eqIdx+1),"UTF-8");
      }
      if(paramKey.equals("id")) {
        hdlReq.hdl = paramVal;
      } else if(passThroughKeys.containsKey(paramKey)) {
        hdlReq.params.addParameter(paramKey, paramVal);
      }
    } while(ampIdx>=0);

    if(hdlReq.hdl==null) hdlReq.hdl = "";
    else hdlReq.hdl = hdlReq.hdl.trim();
    
    hdlReq.normalizeHandle();
    hdlproxy.doResponse(hdlReq);
  }
  
  

  public void doGet(HttpServletRequest request, HttpServletResponse response)
      throws IOException, ServletException {
    doOpenURLResolution(request, response);
  }

  public void doPost(HttpServletRequest request, HttpServletResponse response)
      throws IOException, ServletException {
    doOpenURLResolution(request, response);
  }

}
